************************************************************************
*                                                                      *
*    Datei                                                             *
*      Redox.for                                                       *
*                                                                      *
*    Inhalt                                                            *
*      Redox. Berechnung von beliebigen Redox-Reaktionen.              *
*                                                                      *
*     Entwicklung: Rainer Sieger, Uni Bremen, FB5                      *
*                  Dr. Klaus Wallmann, Uni Bremen, FB5                 *
*                                                                      *
*     letztes Editierdatum: 10-Nov-1992                                *
*                                                                      *
*     Wird aufgerufen in:                                              *
*        Simulation                                                    *
*                                                                      *
*     Function Berechne_y                                              *
*     Wird aufgerufen in:                                              *
*        Redox                                                         *
*                                                                      *
*     Modifiziere_u                                                    *
*     Wird aufgerufen in:                                              *
*        Redox                                                         *
*                                                                      *
*     Function BuCN                                                    *
*     Wird aufgerufen in:                                              *
*        Redox                                                         *
*                                                                      *
************************************************************************
*                                                                      *
*    Copyright                                                         *
*      Der in dieser Datei enthaltene Quellcode darf ohne Genehmigung  *
*      und weitere Vergtung weiterentwickelt werden. Diese Copyright  *
*      Erklrung darf um eigene Urherberrechtserklrungen erweitert    *
*      werden. Die Verwendung dieses Quelltextes fr Publikationen     *
*      ist, auch bei auszugsweiser Verwendung, grundstzlich Genehmi-  *
*      gungspflichtig (kostet nix, ich wrdes halt nur gerne wissen).  *
*                                                                      *
*      Der Programmautor weist darauf hin, da Software nicht fehler-  *
*      frei zu erstellen ist. Die Korrektheit des Quellcodes ist da-   *
*      her auf geeignete Weise zu berprfen.                          *
*                                                                      *
************************************************************************
*                                                                      *
*    Programmautor                                                     *
*      Dr. Rainer Sieger                                               *
*      Alfred-Wegener-Institut fr Meeres- und Polarforschung          *
*      Postfach 12 01 61                                               *
*      27515 Bremerhaven                                               *
*      e-mail: rsieger@AWI-Bremerhaven.de                              *
*                                                                      *
************************************************************************
*                                                                      *
*    letztes Editierdatum                                              *
*      16.3.94                                                         *
*                                                                      *
************************************************************************

***********************************************************************

      Subroutine Redox(n,MS,Anzahl_MS,t,dt,dz,Stoech,UR,CNV,C,
     *                 UMS_Pfad,UMS_File)

**********************************************************************

      Include 'nT.inc'
      Include 'nmax.inc'

      Integer*2       nK
      Parameter      (nK     = nT)      ! Anzahl der kineti. Konstanten

      Integer*2      i, l, k            ! Laufvariablen
      Integer*2      n                  ! # der Zellen
      Integer*2      MS(nT)             ! Masterspezie (Tracer) aktiv ?
      Integer*2      Anzahl_MS          ! Anzahl Masterspezies

      Real*8         Tiefe              ! Ort (Dummy)
      Real*8         dz(nmax)           ! Zellenweite
      Real*8         C(nmax,nT)         ! Konzentration in Zelle i
      Real*8         t                  ! Zeitpunkt
      Real*8         dt                 ! Zeitschrittlnge
      Real*8         Stoech(nT,nK)      ! Stchometrische Faktoren
      Real*8         UR(nmax,nK)        ! Umsatzraten
      Real*8         CNV(nmax)          ! C/N-Verhltnis
      Real*8         CN_V               ! Dummy

      Real*8         u(nK)              ! modifizierte UR
      Real*8         uCN                ! k12
      Real*8         X(nK), Y(nK)       ! Dummys
      Real*8         Summe              ! Dummy

      Real*8         Berechne_y         ! Funktion
      Real*8         BuCN               ! Funktion

      Character*8    Datum              ! Datum beim ffnen des Files
      Character*8    Zeit               ! Zeit beim ffnen des Files
      Character*12   UMS_File           ! Dateiname
      Character*12   AUR_File           ! Dateiname
      Character*30   UMS_Pfad           ! Dateipfad
      Character*42   Filename           ! vollstndiger Dateiname
      Character*80   FT                 ! Filetitel

************************************************************************
*     aktuelles Datum und Zeit ermitteln

      call Datum_Zeit_ermitteln(Datum,Zeit)
      FT = ' CoTAM, aktuelle Umsatzraten, erstellt am '//Datum//' um '

************************************************************************

      if (t.GT.0.D0) then
        AUR_File = UMS_File
        call Setze_Extension(AUR_File,'.AUR')
        call Verbinde_Pfad_Filename(UMS_Pfad,AUR_File,Filename)
        open(unit=5,file=Filename)
        write(5,'(A)') FT
        write(5,'(A,1PE11.4)') ' Zeit :', t
        write(5,'(3X,A)')      ' X    Umsatzraten'
        Tiefe = 0.0
        do i=1,n+2
          do k=1,Anzahl_MS
            u(k) = UR(i,k)
          end do
        end do
      end if

      do i=1,n+2
        Summe = 0.D0
        do k=1,Anzahl_MS
          Summe = Summe + DABS(UR(i,k))
        end do
        if (Summe.GT.0.D0) then   ! Wenn mindestens ein UR(i,l) <> 0
          do k=1,Anzahl_MS
            u(k) = UR(i,k)
            X(k) = 1.D0
          end do
          CN_V = CNV(i)
          uCN  = 0.D0

          do l=1,Anzahl_MS
            if(C(i,l).GT.0.D0) then
              X(l) = -C(i,l)/dt
            else
              X(l) = 0.D0
              do k=1,Anzahl_MS
                if(Stoech(l,k)*u(k).LT.0.D0) then
                  u(k) = 0.D0
                end if
              end do
            end if
          end do

          do l=1,8
            Y(l) = Berechne_y(Anzahl_MS,l,Stoech,u,X(l))
            call Modifiziere_u(Anzahl_MS,l,Y(l),Stoech,u)
          end do

          if(X(9).LT.0.D0) then
            uCN  = BuCN(u,CN_V)
            Y(9) = Berechne_y(Anzahl_MS,9,Stoech,u,X(9))
            Y(9) = Y(9) + uCN/X(9)
            call Modifiziere_u(Anzahl_MS,9,Y(9),Stoech,u)
          end if
          if(X(10).LT.0.D0) then
            uCN   = BuCN(u,CN_V)
            Y(10) = Berechne_y(Anzahl_MS,10,Stoech,u,X(10))
            Y(10) = Y(10) + CN_V*uCN/X(10)
            call Modifiziere_u(Anzahl_MS,10,Y(10),Stoech,u)
          end if
          if(X(11).LT.0.D0) then
            uCN   = BuCN(u,CN_V)
            Y(11) = Berechne_y(Anzahl_MS,11,Stoech,u,X(11))
            Y(11) = Y(11) + uCN*(5.D0*CN_V + 9.D0)/X(11)
            call Modifiziere_u(Anzahl_MS,11,Y(11),Stoech,u)
          end if

          do l=12,Anzahl_MS
            Y(l) = Berechne_y(Anzahl_MS,l,Stoech,u,X(l))
            call Modifiziere_u(Anzahl_MS,l,Y(l),Stoech,u)
          end do

          if (t.LE.0.D0) then
            do l=1,Anzahl_MS
              if (MS(l).GT.0) then
                do k=1,Anzahl_MS
                  C(i,l) = C(i,l) + dt*Stoech(l,k)*u(k)
                end do
              end if
            end do

            uCN     = BuCN(u,CN_V)
            C(i,9)  = C(i,9)  + dt*uCN
            C(i,10) = C(i,10) + dt*CNV(i)*uCN
            C(i,11) = C(i,11) + dt*uCN*(5.D0*CN_V + 9.D0)

            do l=1,Anzahl_MS
              if (C(i,l).LT.0.D0)  C(i,l) = 0.D0
            end do
          end if
        end if
         if (t.GT.0.D0) then
          if (i.EQ.1) then
            Tiefe = dz(1)/2.D0
          else
            Tiefe = Tiefe + dz(i-1)/2.D0 + dz(i)/2.D0
          end if
          write(5,'(F7.1,21(1X,1PE10.3))') Tiefe, (u(l), l=1,Anzahl_MS)
        end if
      end do

      if (t.GT.0)  close(5)

      End

************************************************************************
************************************************************************
************************************************************************

      Real*8 Function Berechne_y(Anzahl_MS,l,Stoech,u,X)

************************************************************************

      Include 'nT.inc'

      Integer*2       nK
      Parameter      (nK     = nT)    ! Anzahl der kineti. Konstanten

      Integer*2      k, l             ! Laufvariablen
      Integer*2      Anzahl_MS        ! Anzahl Masterspezies

      Real*8         Stoech(nT,nK)    ! Stchometrische Faktoren

      Real*8         u(nK)           ! modifizierte UR
      Real*8         X                ! Dummys
      Real*8         Summe            ! Dummy

************************************************************************

      if((X.GT.0.D0) .OR. (X.LT.0.D0)) then
        Summe = 0.D0
        do k=1,Anzahl_MS
          Summe = Summe + Stoech(l,k)*u(k)
        end do
        Berechne_y = Summe/X
      else
        Berechne_y = 0.D0
      end if

      End

************************************************************************
************************************************************************
************************************************************************

      Subroutine Modifiziere_u(Anzahl_MS,l,Y,Stoech,u)

************************************************************************

      Include 'nT.inc'

      Integer*2       nK
      Parameter      (nK     = nT)    ! Anzahl der kineti. Konstanten

      Integer*2      k, l             ! Laufvariablen
      Integer*2      Anzahl_MS        ! Anzahl Masterspezies

      Real*8         Stoech(nT,nK)    ! Stchometrische Faktoren
      Real*8         u(nK)            ! modifizierte UR
      Real*8         Y                ! Dummys

************************************************************************

      if(Y.GT.1.D0) then
        do k=1,Anzahl_MS
          if((Stoech(l,k).GT.0.D0) .OR. (Stoech(l,k).LT.0.D0)) then
            u(k) = u(k)/Y
          end if
        end do
      end if

      End

************************************************************************
************************************************************************
************************************************************************

      Real*8 Function BuCN(u,CNV)

************************************************************************

      Include 'nT.inc'

      Integer*2   nK
      Parameter  (nK = nT)

      Real*8   uCN        ! dummy
      Real*8   u(nK)      ! Umsatzraten
      Real*8   CNV        ! C/N-Verhltnis

************************************************************************

      uCN = 4.D0*u(1) + 2.D0*u(2) + 5.D0*u(3) + 8.D0*u(4)
      uCN = uCN + 3.D0*u(5) + 6.D0*u(6) + 2.D0*u(7)
      uCN = uCN + u(8) + 8.D0*u(9) + 8.D0*u(10)
      BuCN = -uCN/(4.D0*CNV + 8.D0)

      End

